# SPDX-License-Identifier: MIT

from __future__ import annotations

import sys

from typing import Any, Dict, List, Union


if sys.version_info < (3, 11):
    from typing_extensions import Required
else:
    from typing import Required

if sys.version_info < (3, 8):
    from typing_extensions import Literal, TypedDict
else:
    from typing import Literal, TypedDict


__all__ = [
    'ContactTable',
    'LicenseTable',
    'ReadmeTable',
    'ProjectTable',
    'BuildSystemTable',
    'PyProjectTable',
]


def __dir__() -> list[str]:
    return __all__


class ContactTable(TypedDict, total=False):
    name: str
    email: str


class LicenseTable(TypedDict, total=False):
    text: str
    file: str


ReadmeTable = TypedDict(
    'ReadmeTable', {'file': str, 'text': str, 'content-type': str}, total=False
)

ProjectTable = TypedDict(
    'ProjectTable',
    {
        'name': Required[str],
        'version': str,
        'description': str,
        'license': Union[LicenseTable, str],
        'license-files': List[str],
        'readme': Union[str, ReadmeTable],
        'requires-python': str,
        'dependencies': List[str],
        'optional-dependencies': Dict[str, List[str]],
        'entry-points': Dict[str, Dict[str, str]],
        'authors': List[ContactTable],
        'maintainers': List[ContactTable],
        'urls': Dict[str, str],
        'classifiers': List[str],
        'keywords': List[str],
        'scripts': Dict[str, str],
        'gui-scripts': Dict[str, str],
        'dynamic': List[
            Literal[
                'authors',
                'classifiers',
                'dependencies',
                'description',
                'dynamic',
                'entry-points',
                'gui-scripts',
                'keywords',
                'license',
                'maintainers',
                'optional-dependencies',
                'readme',
                'requires-python',
                'scripts',
                'urls',
                'version',
            ]
        ],
    },
    total=False,
)

BuildSystemTable = TypedDict(
    'BuildSystemTable',
    {
        'build-backend': str,
        'requires': List[str],
        'backend-path': List[str],
    },
    total=False,
)

PyProjectTable = TypedDict(
    'PyProjectTable',
    {
        'build-system': BuildSystemTable,
        'project': ProjectTable,
        'tool': Dict[str, Any],
    },
    total=False,
)
